#!/usr/bin/env python

from distutils.core import setup
setup(name='O365',
	version='0.5',
	description='Python library for working with Microsoft Office 365',
	author='Toben Archer',
	author_email='sandslash+O365@gmail.com',
	url='https://github.com/Narcolapser/python-o365',
	packages=['O365']
	)

